$Title GAMS CEM_V5 Model
*Version ESO-XEL myopic
$offlisting
*****************************************************************************
$Ontext
Clara F. Heuberger ~ Imperial College London ~ 09/2017

$Offtext

*****************************************************************************
*Read data from excel speadsheets
*****************************************************************************
* Select problem case.
$set Case 'ESO_Input_RHO_Zenodo'
* Read data from Excel and store in GDX file.
$call gdxxrw %Case%.xlsx skipempty=0 trace=2 index=Specs!A1
$gdxin %Case%.gdx

*****************************************************************************
*Sets
*****************************************************************************
Sets
t                time steps for short-term operation
a                time steps for capacity expansion planning
i                power technologies
ig(i)            power generating technologies
ic(ig)           conventional technologies
ir(ig)           intermittent technologies
is(i)            storage technologies
*il(i)            learning techs
l                segments in LR linearisation
c                cluster set
;

* Load sets from GDX file
$load  t, a, i, ig, ic, ir, is, l, c

* Define alias sets for mode transition constraints
alias(a,a_)
alias(t,t_)

*****************************************************************************
*Parameters
*****************************************************************************
Parameters
LengthT           length of annual scheduling horizon                  h
HoursInYear       number of hours in one year                          h
deltaa            planning period time step width                      years
DIni(i)           number of units of tech installed (designed)         # units
DMax(i)           maximum capacity depoyment until end of a            # units
BR(i)             build rate of tech i                                 # units
LT(i)             lifetime of tech i                                   years
LTIni(i)          lifetime of initial capacity stack                   years
MA(i)             maturity parameter                                   -
Des(i)            capacity per unit of tech i                          MW
TE(i,*)           technical features of tech i in mode m
*TE(i,m,*)         technical features of tech i in mode m
**"Pmin"           minimum power output of tech i                        %-MW
**"Pmax"           maximum power output of tech i                        %-MW
**"IP"             inertia potentail of tech i                           s
**"RP"             reserve potentail of tech i                           %-MW
**"Ems"            emission by tech i                                    tCO2/MWh
**"UtilMin"        minimum utilisation factor                            %-MWh
OPEX(i,a)         opex of tech i in mode m in year a                    per h or MWh
OPEXNL(i)         fixed opex of tech i                                  per h
CAPEX(i)          capex of tech i                                       per # unit
SEta(is)          storage round trip efficiency                        %-MWh
SDur(is)          storage duration                                     h
SOCIni(is)        initial soc level                                    %-MWh
SOCMax(is)        maximum state of charge                              %-MW
SOCMin(is)        minimum state of charge                              %-MW
AV(ir,c,t)        availability factor of techs ir at time t            %-MW
ImpElecPr(c,t)    import electricity price                              per MWh
SD(c,t,a)         system electricity demand in year a at time  t       MWh per h
WF(c)             weighting factor for clusterd hourly data            -
SDTot(a)          total electricity demand in year a                   MWh
SI                system inertia demand at time t                      MW
PL(a)             peak load over time period T                         MW
RM                system reserve margin                                %-MW
WR                system reserve buffer for wind power gen             %-MW wind
TL                network losses                                       %-demand
SE(a)             system carbon emission target                        tCO2 per year
Disc(a)           discount factor for year a                           -
VoLL              value of lost load                                    per MWh
*adjust for single or multi-tech LR
*$Ontext
XloCo(l)            lower x values Coal CCS                              cum MW
XupCo(l)            upper x values Coal CCS                              cum MW
YloCo(l)            lower y values Coal CCS                              cum 
YupCo(l)            upper y values Coal CCS                              cum 
XloCc(l)            lower x values CCGT CCS                              cum MW
XupCc(l)            upper x values CCGT CCS                              cum MW
YloCc(l)            lower y values CCGT CCS                              cum 
YupCc(l)            upper y values CCGT CCS                              cum 
XloBe(l)            lower x values BECCS                                 cum MW
XupBe(l)            upper x values BECCS                                 cum MW
YloBe(l)            lower y values BECCS                                 cum 
YupBe(l)            upper y values BECCS                                 cum 
XloWn(l)            lower x values Onshore Wind                          cum MW
XupWn(l)            upper x values Onshore Wind                          cum MW
YloWn(l)            lower y values Onshore Wind                          cum 
YupWn(l)            upper y values Onshore Wind                          cum 
XloWf(l)            lower x values Offshore Wind                         cum MW
XupWf(l)            upper x values Offshore Wind                         cum MW
YloWf(l)            lower y values Offshore Wind                         cum 
YupWf(l)            upper y values Offshore Wind                         cum 
XloSo(l)            lower x values Solar                                 cum MW
XupSo(l)            upper x values Solar                                 cum MW
YloSo(l)            lower y values Solar                                 cum 
YupSo(l)            upper y values Solar                                 cum 
XloPh(l)            lower x values PHydro                                cum MW
XupPh(l)            upper x values PHydro                                cum MW
YloPh(l)            lower y values PHydro                                cum 
YupPh(l)            upper y values PHydro                                cum 
XloGs(l)            lower x values GenSto                                cum MW
XupGs(l)            upper x values GenSto                                cum MW
YloGs(l)            lower y values GenSto                                cum 
YupGs(l)            upper y values GenSto                                cum 
XloNu(l)            lower x values Nuclear                               cum MW
XupNu(l)            upper x values Nuclear                               cum MW
YloNu(l)            lower y values Nuclear                               cum 
YupNu(l)            upper y values Nuclear                               cum 
XloNT(l)            lower x values NewTech SuperTech                     cum MW
XupNT(l)            upper x values NewTech SuperTech                     cum MW
YloNT(l)            lower y values NewTech SuperTech                     cum 
YupNT(l)            upper y values NewTech SuperTech                     cum 
*SlopeNu(l)          slope nuclear lr curve                               cum 
*$Offtext
$Ontext
Xlo(il,l)            lower x values                                       cum MW
Xup(il,l)            upper x values                                       cum MW
Ylo(il,l)            lower y values                                       cum 
Yup(il,l)            upper y values                                       cum 
$Offtext
*Slope(il,l)          slope of pwl LR curve                                cum  per cum MW
UT(ig)             min up time                                          h
DT(ig)             min down time                                        h
OPEXSU(i)          startup cost                                          per h
;

* Load parameters from GDX file.
$load deltaa, DIni, DMax, BR, LT, Des, TE, OPEX, OPEXNL, OPEXSU, CAPEX, SEta, SDur
$load SOCIni, SOCMax, SOCMin, AV, ImpElecPr, SD, WF, SI, RM, WR, TL, LTIni, MA
$load SE, Disc, VoLL, UT, DT
$load XloCo, XupCo, YloCo, YupCo, XloCc, XupCc, YloCc, YupCc
$load XloBe, XupBe, YloBe, YupBe, XloWn, XupWn, YloWn, YupWn
$load XloWf, XupWf, YloWf, YupWf, XloSo, XupSo, YloSo, YupSo
$load XloPh, XupPh, YloPh, YupPh, XloGs, XupGs, YloGs, YupGs
$load XloNu, XupNu, YloNu, YupNu
$load XloNT, XupNT, YloNT, YupNT
*$load SlopeNu

*Parameter calculations/adjustments
LengthT = card(t);
HoursInYear = 8760;
PL(a) = smax((c,t), SD(c,t,a));
SDTot(a) = sum((c,t), SD(c,t,a));

*****************************************************************************
*Variables
*****************************************************************************
Positive Variables
p(ig,a,c,t)       power output of tech ig in mode m in year a at time t       MWh
p2s(ig,a,c,t)     power to grid-level storage                                 MWh
p2is(is,a,c,t)    power to storage technology is                              MWh
p2d(ig,a,c,t)     power to demand                                             MWh
r(ig,a,c,t)       reserve provided                                            MW
s(is,a,c,t)       state of charge or storage at the END of t                  MWh
s2d(is,a,c,t)     stored energy to demand                                     MWh
s2r(is,a,c,t)     stored energy as reserve capacity                           MW
slak(a,c,t)       slak variable for lost load as feasibility measure          MWh
emslak(a,c,t)     slak variable for carbon emission                           tCO2

*adjust for single or multi-tech LR
xsco(a,l)         position on segment                                         MW
yco(a)            cum capex for in year a                                     
xscc(a,l)         position on segment                                         MW
ycc(a)            cum capex for in year a                                     
xsbe(a,l)         position on segment                                         MW
ybe(a)            cum capex for in year a                                     
xswn(a,l)         position on segment                                         MW
ywn(a)            cum capex for in year a                                     
xswf(a,l)         position on segment                                         MW
ywf(a)            cum capex for in year a                                     
xsso(a,l)         position on segment                                         MW
yso(a)            cum capex for in year a                                     
xsph(a,l)         position on segment                                         MW
yph(a)            cum capex for in year a                                     
xsgs(a,l)         position on segment                                         MW
ygs(a)            cum capex for in year a                                     
xsnu(a,l)         position on segment                                         MW
ynu(a)            cum capex for in year a                                     
xsnt(a,l)         position on segment                                         MW
ynt(a)            cum capex for in year a                                     

u(ig,a,c,t)       up moves                                                   #units
w(ig,a,c,t)       down moves                                                 #units
;


Binary Variable
*adjust for single or multi-tech LR
rhoco(a,l)        one if cum capex of tech i in year a on line segment l      -
rhocc(a,l)        one if cum capex of tech i in year a on line segment l      -
rhobe(a,l)        one if cum capex of tech i in year a on line segment l      -
rhown(a,l)        one if cum capex of tech i in year a on line segment l      -
rhowf(a,l)        one if cum capex of tech i in year a on line segment l      -
rhoso(a,l)        one if cum capex of tech i in year a on line segment l      -
rhoph(a,l)        one if cum capex of tech i in year a on line segment l      -
rhogs(a,l)        one if cum capex of tech i in year a on line segment l      -
rhonu(a,l)        one if cum capex of tech i in year a on line segment l      -
rhont(a,l)        one if cum capex of tech i in year a on line segment l      -
;

Integer Variables
b(i,a)           number of units new built of tech i in year a                # units
d(i,a)           number of operational units of tech i in year a              # units
;

Positive Variables
n(ig,a,c,t)      number of units of tech ig in mode m in year a at time t     # units
o(is,a,c,t)      number of units storage techs is operating at time t         # units
;

*$Ontext
b.lo(i,a) = 0;
b.up(i,a) = DMax(i);

d.lo(i,a) = 0;
d.up(i,a) = DMax(i);

n.lo(ig,a,c,t) = 0;
n.up(ig,a,c,t) = DMax(ig);

o.lo(is,a,c,t) = 0;
o.up(is,a,c,t) = DMax(is);

Variable
tsc            total system cost is objective                               
tse            total system emissions                                       tCO2
e(ig,a,c,t)    emission caused by tech ig                                   tCO2
;

*****************************************************************************
*Constraints
*****************************************************************************
Equations
*system design constraints
DesIni(i,a)
NewB(i,a)
MaxB(i,a)
*TotB(a)
LifeIni(i,a)
LifeI(i,a)
Life(i,a)
UnOn(ig,a,c,t)
StUnOn(is,a,c,t)
*systen-wide constraints
ElecDem(a,c,t)
PeakDem(a)
ResDem(a,c,t)
IneDem(a,c,t)
*EmsTar(a)
UnmetDem(a)

*all generating (ig) tech constraints
PowMin(ic,a,c,t)
PowMax(ig,a,c,t)
PBTech(ig,a,c,t)
EmsIs(ig,a,c,t)
MinUtil(ic,a)

*intermittent renewable (ir) tech constraints
IRMin(ir,a,c,t)
IRavail(ir,a,c,t)


* new UT DT constr
UT1(ic,a,c,t)
DT1(ic,a,c,t)
ST1(ic,a,c,t,t_)
ST2(ic,a,c,t,t_)

*storage (is) tech constraints
NoCharge(ir,a,c,t)

StOn(is,a,c,t)
StMax(is,a,c,t)
StMin(is,a,c,t)
StPow(a,c,t)
SIni(is,a,c,t)
SOC(is,a,c,t)
StDischMax(is,a,c,t)
StMaxTot(is,a,c,t)
StChMax(is,a,c,t)

*tech learning rate consraints = single tech LR
*
$Ontext
SegConvexNu(a)
SegLoNu(a,l)
SegUpNu(a,l)
XValNu(a)
YValNu(a)
*
*$Ontext
*$Offtext
SegConvexCo(a)
SegLoCo(a,l)
SegUpCo(a,l)
XValCo(a)
YValCo(a)
*
*$Offtext
*$Ontext
SegConvexCc(a)
SegLoCc(a,l)
SegUpCc(a,l)
XValCc(a)
YValCc(a)
*
*$Ontext
SegConvexBe(a)
SegLoBe(a,l)
SegUpBe(a,l)
XValBe(a)
YValBe(a)
*
*
SegConvexWn(a)
SegLoWn(a,l)
SegUpWn(a,l)
XValWn(a)
YValWn(a)
*
*$Offtext

SegConvexWf(a)
SegLoWf(a,l)
SegUpWf(a,l)
XValWf(a)
YValWf(a)
*
*$Ontext
SegConvexSo(a)
SegLoSo(a,l)
SegUpSo(a,l)
XValSo(a)
YValSo(a)
*
*$Ontext
SegConvexPh(a)
SegLoPh(a,l)
SegUpPh(a,l)
XValPh(a)
YValPh(a)
*$Offtext
*
SegConvexGs(a)
SegLoGs(a,l)
SegUpGs(a,l)
XValGs(a)
YValGs(a)
*
$Offtext
SegConvexNt(a)
SegLoNt(a,l)
SegUpNt(a,l)
XValNt(a)
YValNt(a)
*$Offtext

$Ontext
*tech learning rate consraints = multiple tech LR
SegConvex(il,a)
SegLo(il,a,l)
SegUp(il,a,l)
XVal(il,a)
YVal(il,a)
$Offtext

*objective function
TotSysCost
TotSysEms(a)
;

**###########################################################################
*=== system design constraints
DesIni(i,a)$(ord(a) eq 1)..     d(i,a) =e= DIni(i);
NewB(i,a)$(ord(a) gt 1)..       b(i,a) =l= BR(i)*MA(i)*deltaa;
MaxB(i,a)..                     d(i,a) =l= DMax(i);
LifeIni(i,a)$(ord(a) le LTIni(i)/deltaa+1).. d(i,a) =e= d(i,a-1) - b(i,a - (LTIni(i)/deltaa)) + b(i,a);
LifeI(i,a)$(LTIni(i)/deltaa+1 lt ord(a) and ord(a) le LT(i)/deltaa+1)..  d(i,a) =e= d(i,a-1) + b(i,a);
Life(i,a)$(ord(a) > LT(i)/deltaa+1)..    d(i,a) =e= d(i,a-1) - b(i,a - (LT(i)/deltaa)) + b(i,a);
UnOn(ig,a,c,t)..                n(ig,a,c,t) =l= d(ig,a);
StUnOn(is,a,c,t)..              o(is,a,c,t) =l= d(is,a);

*=== systen-wide constraints
ElecDem(a,c,t)..   sum(ig, p2d(ig,a,c,t)) + sum(is, s2d(is,a,c,t)) =e= SD(c,t,a)*(1 + TL) - slak(a,c,t);
PeakDem(a)..       sum(i, d(i,a)*Des(i)*TE(i,"Cmax")) =g= PL(a)*(1+RM);
ResDem(a,c,t)..    sum(ig, r(ig,a,c,t)*TE(ig,"Rp")) + sum(is, s2r(is,a,c,t)*TE(is,"Rp")) =g= PL(a)*RM + sum(ir, p2d(ir,a,c,t)*WR);
IneDem(a,c,t)..    sum(ig, n(ig,a,c,t)*Des(ig)*TE(ig,"Ip")) + sum(is, o(is,a,c,t)*Des(is)*TE(is,"Ip")) =g= SI;
*comment constraint EmsTar for purely market/carbon price driven decarbonisation
*EmsTar(a)..        sum((ic,c,t), (e(ic,a,c,t)- emslak(a,c,t))*WF(c)) =l= SE(a);
UnmetDem(a)..      sum((c,t), slak(a,c,t)*WF(c)) =l= 0.005*sum((c,t), SD(c,t,a)*WF(c));


*=== all generating (ig) tech constraints
PowMin(ic,a,c,t)..       p(ic,a,c,t) =g= n(ic,a,c,t)*Des(ic)*TE(ic,"Pmin");
PowMax(ig,a,c,t)..       p(ig,a,c,t) + r(ig,a,c,t) =l= n(ig,a,c,t)*Des(ig)*TE(ig,"Pmax");
PBTech(ig,a,c,t)..       p2d(ig,a,c,t) + p2s(ig,a,c,t) =e= p(ig,a,c,t);
EmsIs(ig,a,c,t)..        e(ig,a,c,t) =e= p(ig,a,c,t)*TE(ig,"Ems");
MinUtil(ic,a)$(ord(a)>1 and ord(a)<8)..   sum((c,t), p(ic,a,c,t)*WF(c)/8760)
                                                 =g= TE(ic,"UtilMin")*d(ic,a)*Des(ic);

*=== intermittent renewable (ir) tech constraints
IRMin(ir,a,c,t)$(ord(ir) le 3)..        p(ir,a,c,t) =g= n(ir,a,c,t)*Des(ir)*TE(ir,"Pmin")*AV(ir,c,t);
IRavail(ir,a,c,t)$(ord(ir) le 3)..      p(ir,a,c,t) + r(ir,a,c,t) =l= n(ir,a,c,t)*Des(ir)*AV(ir,c,t);

*=== new ut dt constraints
UT1(ic,a,c,t)$(ord(t) gt 1).. u(ic,a,c,t) =g= n(ic,a,c,t) - n(ic,a,c,t-1);
DT1(ic,a,c,t)$(ord(t) gt 1).. w(ic,a,c,t) =g= n(ic,a,c,t-1) - n(ic,a,c,t);
ST1(ic,a,c,t,t_)$(ord(t) gt 1 and ord(t_) le UT(ic))..   u(ic,a,c,t) =l= n(ic,a,c,t+(ord(t_)-1));
ST2(ic,a,c,t,t_)$(ord(t) gt 1 and ord(t_) le DT(ic))..   w(ic,a,c,t) =l= d(ic,a) - n(ic,a,c,t+(ord(t_)-1));

*=== storage (is) tech constraints
NoCharge(ir,a,c,t)$(ord(ir) eq 4)..     p2s(ir,a,c,t) =e= 0;

StOn(is,a,c,t)..           s2d(is,a,c,t) + s2r(is,a,c,t) =g= o(is,a,c,t)*Des(is)*TE(is,"Pmin");
StMax(is,a,c,t)..          s(is,a,c,t) =l= o(is,a,c,t)*Des(is)*SOCMax(is)*SDur(is);
StMin(is,a,c,t)$(ord(t) gt SDur(is)).. s(is,a,c,t) =g= o(is,a,c,t)*Des(is)*SOCMin(is)*SDur(is);
StPow(a,c,t)..             sum(ig, p2s(ig,a,c,t)) =e= sum(is, p2is(is,a,c,t));
SIni(is,a,c,t)$(ord(t)=1).. s(is,a,c,t) =e= Des(is)*SOCIni(is)*SDur(is);
SOC(is,a,c,t)$(ord(t) gt 1)..  s(is,a,c,t) =e= s(is,a,c,t-1)
                                          - s2d(is,a,c,t)
                                          + p2is(is,a,c,t)*SEta(is);
StDischMax(is,a,c,t)..       s2d(is,a,c,t) + s2r(is,a,c,t) =l= o(is,a,c,t)*Des(is);
StMaxTot(is,a,c,t)..         s2d(is,a,c,t) + s2r(is,a,c,t) =l= s(is,a,c,t)*SEta(is);
StChMax(is,a,c,t)..          p2is(is,a,c,t) =l= o(is,a,c,t)*Des(is);


*$Ontext
*=== tech learning rate constraints == single tech: define in b
*
$Ontext
SegConvexNu(a)..   sum(l, rhonu(a,l)) =e= 1;
SegLoNu(a,l)..     xsnu(a,l) =g= XloNu(l)*rhonu(a,l);
SegUpNu(a,l)..     xsnu(a,l) =l= XupNu(l)*rhonu(a,l);
XValNu(a)..        sum(a_$(ord(a_) le ord(a)), b("Nuclear",a_)) =e= sum(l, xsnu(a,l));
YValNu(a)..        ynu(a) =e= sum(l, YloNu(l)*rhonu(a,l)
                                     + (YupNu(l)-YloNu(l))/(XupNu(l)-XloNu(l))*(xsnu(a,l) - XloNu(l)*rhonu(a,l)) );
*$Offtext
*$Ontext
SegConvexCo(a)..   sum(l, rhoco(a,l)) =e= 1;
SegLoCo(a,l)..     xsco(a,l) =g= XloCo(l)*rhoco(a,l);
SegUpCo(a,l)..     xsco(a,l) =l= XupCo(l)*rhoco(a,l);
XValCo(a)..        sum(a_$(ord(a_) le ord(a)), b("Coal-PostCCS",a_)) =e= sum(l, xsco(a,l));
YValCo(a)..        yco(a) =e= sum(l, YloCo(l)*rhoco(a,l)
                                     + (YupCo(l)-YloCo(l))/(XupCo(l)-XloCo(l))*(xsco(a,l) - XloCo(l)*rhoco(a,l)) );
*$Offtext
*$Ontext
*$Ontext
SegConvexCc(a)..   sum(l, rhocc(a,l)) =e= 1;
SegLoCc(a,l)..     xscc(a,l) =g= XloCc(l)*rhocc(a,l);
SegUpCc(a,l)..     xscc(a,l) =l= XupCc(l)*rhocc(a,l);
XValCc(a)..        sum(a_$(ord(a_) le ord(a)), b("CCGT-PostCCS",a_)) =e= sum(l, xscc(a,l));
YValCc(a)..        ycc(a) =e= sum(l, YloCc(l)*rhocc(a,l)
                                     + (YupCc(l)-YloCc(l))/(XupCc(l)-XloCc(l))*(xscc(a,l) - XloCc(l)*rhocc(a,l)) );
*
*$Ontext
SegConvexBe(a)..   sum(l, rhobe(a,l)) =e= 1;
SegLoBe(a,l)..     xsbe(a,l) =g= XloBe(l)*rhobe(a,l);
SegUpBe(a,l)..     xsbe(a,l) =l= XupBe(l)*rhobe(a,l);
XValBe(a)..        sum(a_$(ord(a_) le ord(a)), b("BECCS",a_)) =e= sum(l, xsbe(a,l));
YValBe(a)..        ybe(a) =e= sum(l, YloBe(l)*rhobe(a,l)
                                     + (YupBe(l)-YloBe(l))/(XupBe(l)-XloBe(l))*(xsbe(a,l) - XloBe(l)*rhobe(a,l)) );
*$Offtext
*
SegConvexWn(a)..   sum(l, rhown(a,l)) =e= 1;
SegLoWn(a,l)..     xswn(a,l) =g= XloWn(l)*rhown(a,l);
SegUpWn(a,l)..     xswn(a,l) =l= XupWn(l)*rhown(a,l);
XValWn(a)..        sum(a_$(ord(a_) le ord(a)), b("Wind-Onshore",a_)) =e= sum(l, xswn(a,l));
YValWn(a)..        ywn(a) =e= sum(l, YloWn(l)*rhown(a,l)
                                     + (YupWn(l)-YloWn(l))/(XupWn(l)-XloWn(l))*(xswn(a,l) - XloWn(l)*rhown(a,l)) );
*$Offtext
*
SegConvexWf(a)..   sum(l, rhowf(a,l)) =e= 1;
SegLoWf(a,l)..     xswf(a,l) =g= XloWf(l)*rhowf(a,l);
SegUpWf(a,l)..     xswf(a,l) =l= XupWf(l)*rhowf(a,l);
XValWf(a)..        sum(a_$(ord(a_) le ord(a)), b("Wind-Offshore",a_)) =e= sum(l, xswf(a,l));
YValWf(a)..        ywf(a) =e= sum(l, YloWf(l)*rhowf(a,l)
                                     + (YupWf(l)-YloWf(l))/(XupWf(l)-XloWf(l))*(xswf(a,l) - XloWf(l)*rhowf(a,l)) );
*
*$Ontext
SegConvexSo(a)..   sum(l, rhoso(a,l)) =e= 1;
SegLoSo(a,l)..     xsso(a,l) =g= XloSo(l)*rhoso(a,l);
SegUpSo(a,l)..     xsso(a,l) =l= XupSo(l)*rhoso(a,l);
XValSo(a)..        sum(a_$(ord(a_) le ord(a)), b("Solar",a_)) =e= sum(l, xsso(a,l));
YValSo(a)..        yso(a) =e= sum(l, YloSo(l)*rhoso(a,l)
                                     + (YupSo(l)-YloSo(l))/(XupSo(l)-XloSo(l))*(xsso(a,l) - XloSo(l)*rhoso(a,l)) );
*
*$Ontext
SegConvexPh(a)..   sum(l, rhoph(a,l)) =e= 1;
SegLoPh(a,l)..     xsph(a,l) =g= XloPh(l)*rhoph(a,l);
SegUpPh(a,l)..     xsph(a,l) =l= XupPh(l)*rhoph(a,l);
XValPh(a)..        sum(a_$(ord(a_) le ord(a)), b("PHSto",a_)) =e= sum(l, xsph(a,l));
YValPh(a)..        yph(a) =e= sum(l, YloPh(l)*rhoph(a,l)
                                     + (YupPh(l)-YloPh(l))/(XupPh(l)-XloPh(l))*(xsph(a,l) - XloPh(l)*rhoph(a,l)) );
*$Offtext
*
SegConvexGs(a)..   sum(l, rhogs(a,l)) =e= 1;
SegLoGs(a,l)..     xsgs(a,l) =g= XloGs(l)*rhogs(a,l);
SegUpGs(a,l)..     xsgs(a,l) =l= XupGs(l)*rhogs(a,l);
XValGs(a)..        sum(a_$(ord(a_) le ord(a)), b("GenSto",a_)) =e= sum(l, xsgs(a,l));
YValGs(a)..        ygs(a) =e= sum(l, YloGs(l)*rhogs(a,l)
                                     + (YupGs(l)-YloGs(l))/(XupGs(l)-XloGs(l))*(xsgs(a,l) - XloGs(l)*rhogs(a,l)) );
$Offtext
SegConvexNt(a)..   sum(l, rhont(a,l)) =e= 1;
SegLoNt(a,l)..     xsnt(a,l) =g= XloNt(l)*rhont(a,l);
SegUpNt(a,l)..     xsnt(a,l) =l= XupNt(l)*rhont(a,l);
XValNt(a)..        sum(a_$(ord(a_) le ord(a)), b("NewTech",a_)) =e= sum(l, xsnt(a,l));
YValNt(a)..        ynt(a) =e= sum(l, YloNt(l)*rhont(a,l)
                                     + (YupNt(l)-YloNt(l))/(XupNt(l)-XloNt(l))*(xsnt(a,l) - XloNt(l)*rhont(a,l)) );
*$Offtext

$Ontext
*=== tech learning rate constraints == multi tech: define in input sheet
SegConvex(il,a)..   sum(l, rho(il,a,l)) =e= 1;
SegLo(il,a,l)..     xs(il,a,l) =g= Xlo(il,l)*rho(il,a,l);
SegUp(il,a,l)..     xs(il,a,l) =l= Xup(il,l)*rho(il,a,l);
*sum(a_$(ord(a_) le ord(a)), b(il,a_))
XVal(il,a)..        sum(a_$(ord(a_) le ord(a)), b(il,a_)) =e= sum(l, xs(il,a,l));
YVal(il,a)..        y(il,a) =e= sum(l, Ylo(il,l)*rho(il,a,l)
                                     + Slope(il,l)*(xs(il,a,l) - Xlo(il,l)*rho(il,a,l)) );
$Offtext

*=== objective function
* ord(i)<>1   Nuclear
* ord(i)<>6   Coal-PostCCS
* ord(i)<>7   CCGT-PostCCS
* ord(i)<>8   BECCS
* ord(i)<>9   Wind-Onshore
* ord(i)<>10  Wind-Offshore
* ord(i)<>11  Solar
* ord(i)<>12  InterImp
* ord(i)<>13  InterSto
* ord(i)<>14  PHSto
* ord(i)<>15  GenSto
* ord(i)<>16  NewTech
TotSysCost..     tsc =e=  sum((i,a)$(ord(a)<>1
*                                      and ord(i)<>1
*                                       and ord(i)<>6
*                                       and ord(i)<>7
*                                       and ord(i)<>8
*                                       and ord(i)<>9
*                                       and ord(i)<>10
*                                       and ord(i)<>11
*                                       and ord(i)<>14
*                                       and ord(i)<>15
                                       and ord(i)<>16
                                         ), CAPEX(i)*b(i,a)*Des(i)/Disc(a) )
*                          + sum((a), (ynu(a)-ynu(a-1))/Disc(a) )
*                          + sum((a), (yco(a)-yco(a-1))/Disc(a) )
*                          + sum((a), (ycc(a)-ycc(a-1))/Disc(a) )
*                          + sum((a), (ybe(a)-ybe(a-1))/Disc(a) )
*                          + sum((a), (ywn(a)-ywn(a-1))/Disc(a) )
*                          + sum((a), (ywf(a)-ywf(a-1))/Disc(a) )
*                          + sum((a), (yso(a)-yso(a-1))/Disc(a) )
*                          + sum((a), (ygs(a)-ygs(a-1))/Disc(a) )
*                          + sum((a), (yph(a)-yph(a-1))/Disc(a) )
                          + sum((a), (ynt(a)-ynt(a-1))/Disc(a) )
                          + sum((ig,a,c,t), u(ig,a,c,t)*OPEXSU(ig)*WF(c)/Disc(a) )
                          + sum((ig,a,c,t), (OPEX(ig,a)*p(ig,a,c,t)*WF(c) + OPEXNL(ig)*n(ig,a,c,t)*WF(c))/Disc(a) )
                          + sum((is,a,c,t), (OPEX(is,a)*s2d(is,a,c,t)*WF(c) + OPEXNL(is)*o(is,a,c,t)*WF(c))/Disc(a) )
                          + sum((a,c,t), (ImpElecPr(c,t)*p2d("InterImp",a,c,t)*WF(c))/Disc(a) )
* + pslak(a) + rslak(a,t) + islak(a,t)
                          + sum((a,c,t), (slak(a,c,t))*WF(c)*VoLL)
                          + sum((a,c,t), (emslak(a,c,t))*WF(c)*1000000)
                          ;

TotSysEms(a)..   tse(a) =e= sum((ig,c,t), e(ig,a,c,t)*WF(c));

**###########################################################################
*****************************************************************************
*Define model structure
*****************************************************************************
Model CEM_V5 /all/;

$Ontext
**###########################################################################
*****************************************************************************
*Solve model - SINGLE MODEL
*****************************************************************************
option solvelink = 0;
option optCR = 0.03 ;
option iterLim = 1E9 ;
option resLim = 1E5 ;
option threads = 8;
maxexecerror = 1E5;

*$Ontext
*$onecho >cplex.opt
mipemphasis 1
covers 3
cliques 3
PolishAfterEpGap 0.07
EpGap 0.03
*$offecho
CEM_V5.optfile = 1;
*$Offtext

Solve CEM_V5 minimising tsc using mip;

$Offtext
*$Ontext
*****************************************************************************
*Additional Parameters
*****************************************************************************
Parameter
MEP(a,c,t)       marginal electricity price                       per MWh
AvgMEP(a)        average MEP                                      per MWh
MRP(a,c,t)       marginal reserve price                           per MWh
AvgMRP(a)        average MRP                                      per MWh
RL(a,c,t)        reserve level                                   MW
pdtot_an(ig,a)   total annual power to demand                    TWh
s2dtot_an(is,a)  total annual stored power to demand             TWh
p2stot(a)        total power charged to storage                  MWh
pcharg(ig,a)     share of charging power fot tech ig             %
CI_real(a)       total carbon intensity inclu import             tCO2 per MWh
CI_UK(a)         total carbon intensity exclu import             tCO2 per MWh
CD(i,a)          chosen design                                   GW
Util(ig,a)       utilisation of capacity                         %
OPEX_SU          total conv. tech opex start up                  
OPEX_INC_G       total generation techs opex                     
OPEX_INC_S       total storage opex                              
OPEX_IMP         total import cost                               
CAPEX_TOT(i,a)   total capex per tech in year                    
CAPEX_Fix_All    total capex in year if tech cost are static     
CAPEX_Learn_All  total capex in year if techs are learning       
Invest_TOT(i,a)  total investment cost per tech                  
CostIG_TOT(ig,a) total capex and opex cost ig techs              
CostIS_TOT(is,a) total capex and opex cost is techs              
CostImp_TOT(a)   total capex and opex cost InterImp              
IRESCurt(ir,a)   curtailed ir power                              MWh
UnmetSD_TOT(a)   total unmet demand                              MWh
Cost_UnmetSD     total cost unment demand                        
TSC_an(a)        total system cost annual                        bn 
;

*****************************************************************************
*Solve model - ROLLING HORIZON APPROACH
*****************************************************************************
*$Ontext
Set
$Ontext
h                rolling horizon interations      /iter1*iter2/
relaxed(h,a)     integer relaxed part of horizon /iter1.(5*8)/
fixed(h,a)       integer fixed part of horizon   /iter2.(1*4)/

$Offtext

h                rolling horizon interations     /iter1*iter7/
$Ontext
relaxed(h,a)     integer relaxed part of horizon /iter1.(3*8)
                                                  iter2.(7*8)/
fixed(h,a)       integer fixed part of horizon   /iter2.(1*2)
                                                  iter3.(1*6)/

$Offtext
*$Ontext

relaxed(h,a)     integer relaxed part of horizon /iter1.(3*8)
                                                  iter2.(4*8)
                                                  iter3.(5*8)
                                                  iter4.(6*8)
                                                  iter5.(7*8)
                                                  iter6.(8)/
fixed(h,a)       integer fixed part of horizon   /iter2.(1*2)
                                                  iter3.(1*3)
                                                  iter4.(1*4)
                                                  iter5.(1*5)
                                                  iter6.(1*6)
                                                  iter7.(1*7)/
*$Offtext
$Ontext

relaxed(h,a)     integer relaxed part of horizon /iter1.(2*8)
                                                  iter2.(3*8)
                                                  iter3.(4*8)
                                                  iter4.(5*8)
                                                  iter5.(6*8)
                                                  iter6.(7*8)
                                                  iter7.(8)/
fixed(h,a)       integer fixed part of horizon   /iter2.(1)
                                                  iter3.(1*2)
                                                  iter4.(1*3)
                                                  iter5.(1*4)
                                                  iter6.(1*5)
                                                  iter7.(1*6)
                                                  iter8.(1*7)/
$Offtext
;
Parameter
MA_h(i,h)        technology maturity factor
;
$load MA_h

************************
*Solve options
************************
CEM_V5.prioropt = 1 ;
option solvelink = 0;
option optCR = 0.03 ;
option iterLim = 1E9 ;
option resLim = 1E5 ;
option threads = 8;
maxexecerror = 1E5;
************************
loop(h,
d.prior(i,a)=1.0;
b.prior(i,a)=1.0;
  loop(relaxed(h,a),
       d.prior(i,a)=INF;
       b.prior(i,a)=INF;
       );
  loop(fixed(h,a),
       d.fx(i,a)=d.l(i,a);
       b.fx(i,a)=b.l(i,a);
       );
*reinitialise technology availability
MA(i) = MA_h(i,h);

************SOLVE CEM_V5 with MIPSTART from RHO
$onecho >cplex.opt
mipemphasis 1
covers 3
cliques 3
PolishAfterEpGap 0.07
EpGap 0.03
$offecho
CEM_V5.optfile = 1;
Solve CEM_V5 minimising tsc using mip;
************

*d.lo(i,a)=0;
*d.up(i,a)=DMax(i);

*b.lo(i,a)=0;
*b.up(i,a)=DMax(i);

);
*$Offtext

$Ontext
************SOLVE CEM_V5 with MIPSTART from RHO
*$onecho >cplex.opt
mipstart 1
mipemphasis 1
covers 3
cliques 3
PolishAfterEpGap 0.07
EpGap 0.03
*$offecho
CEM_V5.optfile = 1;
Solve CEM_V5 minimising tsc using mip;
************
$Offtext

*****************************************************************************
*Evaluate Additional Parameters
*****************************************************************************
*$Ontext
MEP(a,c,t) = ElecDem.m(a,c,t)/WF(c);
AvgMEP(a) = sum((c,t), MEP(a,c,t)*WF(c))/8760;
MRP(a,c,t) = ResDem.m(a,c,t)/WF(c);
AvgMRP(a) = sum((c,t), MRP(a,c,t)*WF(c))/8760;
RL(a,c,t) = PL(a)*RM + sum(ir, p.l(ir,a,c,t)*WR);
pdtot_an(ig,a) = sum((c,t), p2d.l(ig,a,c,t)*WF(c))/1000000;
s2dtot_an(is,a) = sum((c,t), s2d.l(is,a,c,t)*WF(c))/1000000;
p2stot(a) = sum((ig,c,t), p2s.l(ig,a,c,t)*WF(c));
pcharg(ig,a) = sum((c,t), p2s.l(ig,a,c,t)*WF(c))/p2stot(a);
CI_real(a) = tse.l(a)/sum((ig,c,t), p.l(ig,a,c,t)*WF(c));
CI_UK(a) =  sum((ic,c,t), (e.l(ic,a,c,t))*WF(c))/sum((ig,c,t), p.l(ig,a,c,t)*WF(c));
CD(i,a) = d.l(i,a)*Des(i)/1000;
Util(ig,a)$(d.l(ig,a)>0) = sum((c,t), p.l(ig,a,c,t)*WF(c)/8760)/(d.l(ig,a)*Des(ig));
OPEX_SU =  sum((ig,a,c,t), u.l(ig,a,c,t)*OPEXSU(ig)*WF(c)/Disc(a) );
OPEX_INC_G = sum((ig,a,c,t), ( OPEX(ig,a)*p.l(ig,a,c,t)*WF(c) + OPEXNL(ig)*n.l(ig,a,c,t)*WF(c) )/Disc(a) );
OPEX_INC_S = sum((is,a,c,t), ( OPEX(is,a)*s2d.l(is,a,c,t)*WF(c)  + OPEXNL(is)*o.l(is,a,c,t)*WF(c) )/Disc(a) );
OPEX_IMP = sum((a,c,t), ImpElecPr(c,t)*p2d.l("InterImp",a,c,t)*WF(c)/Disc(a) );
CAPEX_TOT(i,a) = CAPEX(i)*d.l(i,a)*Des(i)/Disc(a);
Invest_TOT(i,a) = CAPEX(i)*b.l(i,a)*Des(i)/Disc(a);
CAPEX_Fix_All = sum((i,a)$(ord(a)<>1), CAPEX(i)*b.l(i,a)*Des(i)/Disc(a) );
*CAPEX_Learn_All = sum((i,a)$(ord(a)<>1
*                               and ord(i)<>6 and ord(i)<>7
*                               and ord(i)<>8 and ord(i)<>9 and ord(i)<>11
*                               and ord(i)<>10 and ord(i)<>15 and ord(i)<>16
*                                ), CAPEX(i)*b.l(i,a)*Des(i)/Disc(a)
*                          + (yco.l(a)-yco.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ycc.l(a)-ycc.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ybe.l(a)-ybe.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ywn.l(a)-ywn.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ywf.l(a)-ywf.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (yso.l(a)-yso.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ygs.l(a)-ygs.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ynt.l(a)-ynt.l(a-1))/Disc(a)$(ord(a)<>1));
*CostIG_TOT(ig,a)$(ord(ig)<>12) =  CAPEX_TOT(ig,a) + OPEX_SU(ig,a) + OPEX_INC_G(ig,a);
*CostIS_TOT(is,a) =  CAPEX_TOT(is,a) + OPEX_INC_S(is,a);
*CostImp_TOT(a) =  CAPEX_TOT("InterImp",a) + sum((c,t), (ImpElecPr(c,t)*p2d.l("InterImp",a,c,t)*WF(c))/Disc(a) );
IRESCurt(ir,a)$(ord(ir)<>4) = sum((c,t), (d.l(ir,a)*Des(ir)*AV(ir,c,t) - p.l(ir,a,c,t))*WF(c)) / sum((c,t), d.l(ir,a)*Des(ir)*AV(ir,c,t)*WF(c) );
UnmetSD_TOT(a) = sum((c,t), slak.l(a,c,t)*WF(c))/sum((c,t), SD(c,t,a)*(1 + TL)*WF(c));
Cost_UnmetSD = sum((a,c,t), slak.l(a,c,t)*WF(c)*VoLL/Disc(a) );
TSC_an(a) = sum((i)$(ord(a)<>1
*                               and ord(i) <>1
*                               and ord(i)<>6 and ord(i)<>7
*                               and ord(i)<>8 and ord(i)<>9 and ord(i)<>11
*                               and ord(i)<>10 and ord(i)<>15
                                and ord(i)<>16
                               ), CAPEX(i)*b.l(i,a)*Des(i)/Disc(a))
*                          + (ynu.l(a)-ynu.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (yco.l(a)-yco.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ycc.l(a)-ycc.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ybe.l(a)-ybe.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ywn.l(a)-ywn.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ywf.l(a)-ywf.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (yso.l(a)-yso.l(a-1))/Disc(a)$(ord(a)<>1)
*                          + (ygs.l(a)-ygs.l(a-1))/Disc(a)$(ord(a)<>1)
                          + (ynt.l(a)-ynt.l(a-1))/Disc(a)$(ord(a)<>1)
                          + sum((ig,c,t), u.l(ig,a,c,t)*OPEXSU(ig)*WF(c)/Disc(a) )
                          + sum((ig,c,t), (OPEX(ig,a)*p.l(ig,a,c,t)*WF(c) + OPEXNL(ig)*n.l(ig,a,c,t)*WF(c))/Disc(a) )
                          + sum((is,c,t), (OPEX(is,a)*s2d.l(is,a,c,t)*WF(c) + OPEXNL(is)*o.l(is,a,c,t)*WF(c))/Disc(a) )
                          + sum((c,t), (ImpElecPr(c,t)*p2d.l("InterImp",a,c,t)*WF(c))/Disc(a) )
* + pslak.l(a) + rslak.l(a,t) + islak.l(a,t)
                          + sum((c,t), (slak.l(a,c,t))*WF(c)*VoLL/Disc(a) )
*                          + sum((c,t), (emslak.l(a,c,t))*WF(c)*1000)
                          ;
*$Offtext

*****************************************************************************
*Excel output
*****************************************************************************
$set tech L
$set xlsFile 8a11cl_2a_lr2ST_Ztest
Execute_Unload "ESOutRHO_%tech%_%xlsFile%.gdx";
